# 7 Segment Chars Demo - New driver
# Waveshare Pico LCD-1.8" SPI Display
# Tony Goodhew 14th July 2021
from machine import Pin,SPI,PWM
import framebuf
import utime

BL = 13   # Pins used by SPI driver  
DC = 8
RST = 12
MOSI = 11
SCK = 10
CS = 9

# New driver received from WaveShare on 10th July 2021
class LCD_1inch8(framebuf.FrameBuffer):
    def __init__(self):
        self.width = 160
        self.height = 128        
        self.cs = Pin(CS,Pin.OUT)
        self.rst = Pin(RST,Pin.OUT)        
        self.cs(1)
        self.spi = SPI(1)
        self.spi = SPI(1,1000_000)
        self.spi = SPI(1,10000_000,polarity=0, phase=0,sck=Pin(SCK),mosi=Pin(MOSI),miso=None)
        self.dc = Pin(DC,Pin.OUT)
        self.dc(1)
        self.buffer = bytearray(self.height * self.width * 2)
        super().__init__(self.buffer, self.width, self.height, framebuf.RGB565)
        self.init_display()
        
        self.WHITE  = 0xFFFF
        self.BLACK  = 0x0000
        self.GREEN  = 0x001F
        self.BLUE   = 0xF800
        self.RED    = 0x07E0
        
    def write_cmd(self, cmd):
        self.cs(1)
        self.dc(0)
        self.cs(0)
        self.spi.write(bytearray([cmd]))
        self.cs(1)

    def write_data(self, buf):
        self.cs(1)
        self.dc(1)
        self.cs(0)
        self.spi.write(bytearray([buf]))
        self.cs(1)

    def init_display(self):
        """Initialize display"""  
        self.rst(1)
        self.rst(0)
        self.rst(1)        
        self.write_cmd(0x36);
        self.write_data(0x70);        
        self.write_cmd(0x3A);
        self.write_data(0x05);
         #ST7735R Frame Rate
        self.write_cmd(0xB1);
        self.write_data(0x01);
        self.write_data(0x2C);
        self.write_data(0x2D);
        self.write_cmd(0xB2);
        self.write_data(0x01);
        self.write_data(0x2C);
        self.write_data(0x2D);
        self.write_cmd(0xB3);
        self.write_data(0x01);
        self.write_data(0x2C);
        self.write_data(0x2D);
        self.write_data(0x01);
        self.write_data(0x2C);
        self.write_data(0x2D);
        self.write_cmd(0xB4); #Column inversion
        self.write_data(0x07);
        #ST7735R Power Sequence
        self.write_cmd(0xC0);
        self.write_data(0xA2);
        self.write_data(0x02);
        self.write_data(0x84);
        self.write_cmd(0xC1);
        self.write_data(0xC5);
        self.write_cmd(0xC2);
        self.write_data(0x0A);
        self.write_data(0x00);
        self.write_cmd(0xC3);
        self.write_data(0x8A);
        self.write_data(0x2A);
        self.write_cmd(0xC4);
        self.write_data(0x8A);
        self.write_data(0xEE);
        self.write_cmd(0xC5); #VCOM
        self.write_data(0x0E);
        #ST7735R Gamma Sequence
        self.write_cmd(0xe0);
        self.write_data(0x0f);
        self.write_data(0x1a);
        self.write_data(0x0f);
        self.write_data(0x18);
        self.write_data(0x2f);
        self.write_data(0x28);
        self.write_data(0x20);
        self.write_data(0x22);
        self.write_data(0x1f);
        self.write_data(0x1b);
        self.write_data(0x23);
        self.write_data(0x37);
        self.write_data(0x00);
        self.write_data(0x07);
        self.write_data(0x02);
        self.write_data(0x10);
        self.write_cmd(0xe1);
        self.write_data(0x0f);
        self.write_data(0x1b);
        self.write_data(0x0f);
        self.write_data(0x17);
        self.write_data(0x33);
        self.write_data(0x2c);
        self.write_data(0x29);
        self.write_data(0x2e);
        self.write_data(0x30);
        self.write_data(0x30);
        self.write_data(0x39);
        self.write_data(0x3f);
        self.write_data(0x00);
        self.write_data(0x07);
        self.write_data(0x03);
        self.write_data(0x10);
        self.write_cmd(0xF0); #Enable test command
        self.write_data(0x01);
        self.write_cmd(0xF6); #Disable ram power save mode
        self.write_data(0x00);
            #sleep out
        self.write_cmd(0x11);
        #DEV_Delay_ms(120);
        #Turn on the LCD display
        self.write_cmd(0x29);
    def show(self):
        self.write_cmd(0x2A)
        self.write_data(0x00)
        self.write_data(0x01)
        self.write_data(0x00)
        self.write_data(0xA0)       
        self.write_cmd(0x2B)
        self.write_data(0x00)
        self.write_data(0x02)
        self.write_data(0x00)
        self.write_data(0x82)        
        self.write_cmd(0x2C)        
        self.cs(1)
        self.dc(1)
        self.cs(0)
        self.spi.write(self.buffer)
        self.cs(1)
# ==============End of driver code==============================
def colour(R,G,B):
# Get RED value
    rp = int(R*31/255) # range 0 to 31
    if rp < 0: rp = 0
    r = rp *8
# Get Green value - more complicated!
    gp = int(G*63/255) # range 0 - 63
    if gp < 0: gp = 0
    g = 0
    if gp & 1:  g = g + 8192
    if gp & 2:  g = g + 16384
    if gp & 4:  g = g + 32768
    if gp & 8:  g = g + 1
    if gp & 16: g = g + 2
    if gp & 32: g = g + 4
# Get BLUE value       
    bp =int(B*31/255) # range 0 - 31
    if bp < 0: bp = 0
    b = bp *256
    colour = r+g+b
    return colour

# Set up the display
pwm = PWM(Pin(BL))
pwm.freq(1000)
#pwm.duty_u16(32768)#max 65535
pwm.duty_u16(32768)#max 65535
LCD = LCD_1inch8()
# Background colour
LCD.fill(LCD.BLACK)
LCD.show()
#============= MAIN Program ===============
nums =[1,1,1,1,1,1,0,  # 0 # One row per digit 
       0,1,1,0,0,0,0,  # 1
       1,1,0,1,1,0,1,  # 2
       1,1,1,1,0,0,1,  # 3
       0,1,1,0,0,1,1,  # 4
       1,0,1,1,0,1,1,  # 5
       1,0,1,1,1,1,1,  # 6
       1,1,1,0,0,0,0,  # 7
       1,1,1,1,1,1,1,  # 8
       1,1,1,0,0,1,1,  # 9
       1,1,1,1,1,0,1,  # a = 10 - HEX characters
       0,0,1,1,1,1,1,  # b = 11
       0,0,0,1,1,0,1,  # c = 12
       0,1,1,1,1,0,1,  # d = 13
       1,1,0,1,1,1,1,  # e = 14
       1,0,0,0,1,1,1,  # f = 15
       1,1,1,1,0,1,1,  # g needed for seg!
       0,0,0,0,0,0,1,  # -
       0,0,0,0,0,0,0]  # Blank

def seg(xx,yy,n,f,bg,fg):
    # (x, y, number, size-factor, background, foreground)
    c = [bg,fg]
    p = n * 7    
    LCD.fill_rect(xx+1*f,yy+0*f,3*f,1*f,c[nums[p]])
    LCD.fill_rect(xx+4*f,yy+1*f,1*f,3*f,c[nums[p+1]])
    LCD.fill_rect(xx+4*f,yy+5*f,1*f,3*f,c[nums[p+2]])
    LCD.fill_rect(xx+1*f,yy+8*f,3*f,1*f,c[nums[p+3]])
    LCD.fill_rect(xx+0*f,yy+5*f,1*f,3*f,c[nums[p+4]])
    LCD.fill_rect(xx+0*f,yy+1*f,1*f,3*f,c[nums[p+5]])
    LCD.fill_rect(xx+1*f,yy+4*f,3*f,1*f,c[nums[p+6]])   
    LCD.show()

LCD.fill(0x0)
LCD.rect(0,0,160,128,colour(0,0,255)) # Blue Frame
# White Corner pixels
LCD.pixel(0,0,0xFFFF)     # LT
LCD.pixel(0,127,0xFFFF)   # LB
LCD.pixel(159,0,0xFFFF)   # RT
LCD.pixel(159,127,0xFFFF) # RB
LCD.text("WaveShare", 38,10,colour(255,0,0))
LCD.text('Pico Display 1.8"', 10,23,colour(0,255,0))
LCD.text("7-Seg characters",16,100,colour(255,0,0))
LCD.text("Tony Goodhew",30,113,colour(0,0,255))

top = 40
top2 = top+12
left = 23
c = colour(255,255,0) # yellow
seg(left+5,top,7,5,0,c)
seg(left+30,top,17,5,0,c)
seg(left+54,top2,5,3,0,c)
seg(left+72,top2+12,14,2,0,c)
seg(left+84,top2+12,16,3,0,c)    
utime.sleep(2.5)

x0 = 2
y0 = 10
for f in range(1,10):
    # Adjust Top Left position of number
    x1 = x0 + (9-f) *4
    y1 = y0 + (9-f)
    LCD.fill(0x0)
    LCD.text("7-Segment Size: "+str(f),10,100,colour(255,255,0))
    LCD.text("Tony Goodhew",30,113,colour(0,0,200))
    LCD.show()
    for n in range(20):
        nn = n + (f-1)*20 + (f-1)*100
        
        hnds = int(nn/100)
        nn = nn - (hnds * 100)
        tens = int(nn / 10)
        units = nn - (tens * 10)
        seg(x1,y1,hnds,f,0,0xF)
        seg(x1 + f*6,y1,tens,f,0,0xF)
        seg(x1 + f*12,y1,units,f,0,0xF)
        LCD.show()
utime.sleep(1.5)

LCD.fill(0x0)
LCD.text("Hexadecimal plus",10,100,colour(255,255,0))
LCD.text("Tony Goodhew",30,113,colour(0,0,255))
for n in range(0,19):
    seg(60,25,n,6,0,0xF)
    utime.sleep(1)
LCD.fill(0x0)
LCD.show()



